/*!
 * Buession buession-shirojs v1.0.2
 *
 * @link https://buession.github.io/buession-shirojs/
 * @source https://github.com/buession/buession-shirojs
 * @copyright @ 2020-2021 Buession.com Inc.
 * @license MIT
 * @Build Time Tue, 07 Dec 2021 14:00:03 GMT
 */

(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.Shiro = factory());
})(this, (function () { 'use strict';

	var ALL_PERMISSION = ':';

	var hasAny = function (data, value, all) {
	    return typeof data !== 'undefined' && data != null ? data.indexOf(value) >= 0 : false;
	};
	var hasAll = function (data, values, all) {
	    if (typeof data !== 'undefined' && data != null) {
	        for (var _i = 0, values_1 = values; _i < values_1.length; _i++) {
	            var value = values_1[_i];
	            if (data.indexOf(value) < 0) {
	                return false;
	            }
	        }
	        return true;
	    }
	    return false;
	};

	var Shiro = /** @class */ (function () {
	    function Shiro(principal) {
	        this.principal = principal;
	    }
	    /**
	     * 验证是否为已认证通过的用户，不包含已记住的用户，这是与 isUser 标签方法的区别所在
	     *
	     * @return 用户是否已通过认证
	    */
	    Shiro.prototype.isAuthenticated = function () {
	        return this.principal !== null;
	    };
	    /**
	     * 验证是否为未认证通过用户，与 isAuthenticated 标签相对应，与 isGuest 标签的区别是，该标签包含已记住用户
	     *
	     * @return 用户是否未通过认证
	     */
	    Shiro.prototype.isNotAuthenticated = function () {
	        return this.isAuthenticated() === false;
	    };
	    /**
	     * 验证用户是否为访客，即未认证（包含未记住）的用户
	     *
	     * @return 用户是否为访客
	     */
	    Shiro.prototype.isGuest = function () {
	        return this.principal === null;
	    };
	    /**
	     * 验证用户是否认证通过或已记住的用户
	     *
	     * @return 用户是否认证通过或已记住的用户
	     */
	    Shiro.prototype.isUser = function () {
	        return this.principal !== null;
	    };
	    /**
	     * 验证用户是否具备某角色。
	     *
	     * @param roleName
	     * 		角色名称
	     *
	     * @return 用户是否具备某角色
	     */
	    Shiro.prototype.hasRole = function (roleName) {
	        return this.isAuthenticated() && hasAny(this.principal.getRoles(), roleName);
	    };
	    /**
	     * 验证用户是否不具备某角色，与 hasRole 逻辑相反
	     *
	     * @param roleName
	     * 		角色名称
	     *
	     * @return 用户是否不具备某角色
	     */
	    Shiro.prototype.lacksRole = function (roleName) {
	        return this.hasRole(roleName) === false;
	    };
	    /**
	     * 验证用户是否具有以下任意一个角色
	     *
	     * @param roleNames
	     * 		角色列表
	     *
	     * @return 用户是否具有以下任意一个角色
	     */
	    Shiro.prototype.hasAnyRole = function (roleNames) {
	        if (this.isAuthenticated() == false) {
	            return false;
	        }
	        for (var _i = 0, roleNames_1 = roleNames; _i < roleNames_1.length; _i++) {
	            var roleName = roleNames_1[_i];
	            if (hasAny(this.principal.getRoles(), roleName)) {
	                return true;
	            }
	        }
	        return false;
	    };
	    /**
	     * 验证用户是否具有以下所有角色。
	     *
	     * @param roleNames
	     * 		角色列表
	     *
	     * @return 用户是否具有以下所有角色
	     */
	    Shiro.prototype.hasRolesAll = function (roleNames) {
	        return this.isAuthenticated() && hasAll(this.principal.getRoles(), roleNames);
	    };
	    /**
	     * 验证用户是否具备某权限
	     *
	     * @param permission
	     * 		权限名称
	     *
	     * @return 用户是否具备某权限
	     */
	    Shiro.prototype.hasPermission = function (permission) {
	        return this.isAuthenticated() && (hasAny(this.principal.getPermissions(), permission) || hasAny(this.principal.getPermissions(), ALL_PERMISSION));
	    };
	    /**
	     * 验证用户是否不具备某权限，与 hasPermission 逻辑相反
	     *
	     * @param permission
	     * 		权限名称
	     *
	     * @return 用户是否不具备某权限
	     */
	    Shiro.prototype.lacksPermission = function (permission) {
	        return this.hasPermission(permission) === false;
	    };
	    /**
	     * 验证用户是否具有以下任意一个权限
	     *
	     * @param permissions
	     * 		权限列表
	     *
	     * @return 用户是否具有以下任意一个权限
	     */
	    Shiro.prototype.hasAnyPermission = function (permissions) {
	        if (this.isAuthenticated() == false) {
	            return false;
	        }
	        if (hasAny(this.principal.getPermissions(), ALL_PERMISSION)) {
	            return true;
	        }
	        for (var _i = 0, permissions_1 = permissions; _i < permissions_1.length; _i++) {
	            var permission = permissions_1[_i];
	            if (hasAny(this.principal.getPermissions(), permission)) {
	                return true;
	            }
	        }
	        return false;
	    };
	    /**
	     * 验证用户是否具有以下所有权限
	     *
	     * @param permissions
	     * 		权限列表
	     *
	     * @return 用户是否具有以下所有权限
	     */
	    Shiro.prototype.hasPermissionsAll = function (permissions) {
	        return hasAny(this.principal.getPermissions(), ALL_PERMISSION) || hasAll(this.principal.getPermissions(), permissions);
	    };
	    return Shiro;
	}());

	return Shiro;

}));
//# sourceMappingURL=shirojs.umd.js.map
